""""Collect addition debug information for issue #896"""

import pathlib
import subprocess
import sys
from tempfile import TemporaryDirectory

import click
from rich import print

import osxphotos
from osxphotos import PhotosDB
from osxphotos.exiftool import ExifTool
from osxphotos.photoexporter import ExportOptions, PhotoExporter
from osxphotos.utils import _get_os_version


def get_filetype(file: str):
    """Return file type of file as returned by file command"""

    cmd = ["file", "-b", file]
    result = subprocess.run(cmd, capture_output=True)
    if result.returncode == 0:
        return result.stdout.decode("utf-8").strip()
    else:
        return None


def check_extension(filepath: str) -> tuple[bool, str, str]:
    """Check if file extension is correct for image file using exiftool

    Args:
        filepath: path to file to check

    Returns: tuple of (bool, str, str) where bool is True if extension is correct, False if not
             and str, str is the current extension, correct extension or current extension if correct
    """
    filepath = pathlib.Path(filepath)
    current_ext = filepath.suffix.lower()

    current_ext = current_ext[1:] if current_ext else ""  # remove leading dot
    exiftool = ExifTool(filepath)
    correct_ext = exiftool.asdict().get("File:FileTypeExtension").lower()
    if current_ext != correct_ext:
        # there are some extensions that have more than one valid extension
        # there are likely more but these are the ones I've seen so far
        is_correct = (
            current_ext in ("jpg", "jpeg") and correct_ext in ("jpg", "jpeg")
        ) or (current_ext in ("tif", "tiff") and correct_ext in ("tif", "tiff"))
    else:
        is_correct = True
    return is_correct, current_ext, correct_ext


@click.command()
@click.option(
    "--uuid",
    "uuids",
    default=None,
    multiple=True,
    required=True,
    help="UUID of photo to debug",
)
def main(uuids):
    """Collect debug info for issue #896"""
    photosdb = PhotosDB()

    print(f"OS: {_get_os_version()}")
    print(f"Python: {sys.version}")
    print(f"osxphotos: {osxphotos.__version__}")
    print(f"Library: {photosdb.library_path}")

    for uuid in uuids:
        if photo := photosdb.get_photo(uuid):
            print("*"*20, f" UUID: {photo.uuid} ", "*"*20)
            info = photo._info.copy()
            # remove some fields that may be private
            del info["reverse_geolocation"]
            del info["longitude"]
            del info["latitude"]
            print(info)
            # check paths already on disk
            for path_component in ["path", "path_edited", "path_live_photo"]:
                if path := getattr(photo, path_component):
                    print(f"{path_component}: {path}")
                    print(f"file type: {get_filetype(path)}")
            # check paths as exported with photokit
            with TemporaryDirectory() as tmpdir:
                print(f"exporting files with photokit to {tmpdir}")
                exporter = PhotoExporter(photo)
                export_options = ExportOptions(
                    use_photos_export=True, use_photokit=True, live_photo=True
                )
                results = exporter.export(tmpdir, options=export_options)
                if results.error:
                    print(f"export error: {results.error}")
                for path in results.exported:
                    print(f"exported: {path}")
                    print(f"file type: {get_filetype(path)}")
                    print(f"extension correct: {check_extension(path)}")
                if photo.hasadjustments:
                    export_options = ExportOptions(
                        use_photos_export=True,
                        use_photokit=True,
                        live_photo=True,
                        edited=True,
                    )
                    results = exporter.export(tmpdir, export_options)
                    if results.error:
                        print(f"export error: {results.error}")
                    for path in results.exported:
                        print(f"exported edited: {path}")
                        print(f"file type: {get_filetype(path)}")
                        print(f"extension correct: {check_extension(path)}")


if __name__ == "__main__":
    main()
